/*
 *  Part of rvglue source code,
 *  Copyright (C) 2000 Alexander Kroeller (alilein@gmx.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "appopts.h"
#include "errdesc.h"

/* definition of extern object in appopts.h */
AppOptions appopt;




AppOptions::AppOptions()
{
  msglevel=1;
  errhelp=true;
  woutname=ncpoutname=inputfname=NULL;
}

AppOptions::~AppOptions()
{
  if(woutname != NULL)
    free(woutname);
  if(ncpoutname != NULL)
    free(ncpoutname);
}

void AppOptions::usage(void)
{
  printf("rvglue version %s\n\n",APPVERSION);
  
  printf("Usage: rvglue [options] description-file\n");
  printf("where options are:\n");

  printf("  -o=filename, -output=filename\n");
  printf("                writes to <filename>.w and <filename>.ncp,\n");
  printf("                ignores \"create\" statement.\n");
  //  printf("  -f, -final    Does lengthy optimizations before writing\n");
  printf("  -v, -verbose  Prints what is currently being done\n");
  printf("  -d, -debug    Prints lots of information while working\n");
  printf("  -q, -quiet    Suppresses most output\n");
  printf("  -n, -nohelp   Suppresses printing of help texts when\n");
  printf("                reporting errors.\n");
  printf("  -h, -help     Print this help message and exit.\n");
}


void AppOptions::setOutName(const char *basename)
{
  int blgt;

  if(woutname == NULL)
    {
      blgt=strlen(basename);
      woutname=(char*)malloc(blgt + 3);
      ncpoutname=(char*)malloc(blgt + 5);
      strcpy(woutname,basename); strcat(woutname,".w");
      strcpy(ncpoutname,basename); strcat(ncpoutname,".ncp");
    };
}


bool AppOptions::trySetOutName(char *fn)
{
  if(woutname != NULL)
    {
      perror(0,ERROR_DUPLICATE_OFILE,"Multiple output file names ('%s' and '%s')",woutname,fn);
      return false;
    }
  else if(fn[0]==0)
    {
      perror(0,ERROR_EMPTY_OFILE,"Empty output file name");
      return false;
    }
  else
    {
      setOutName(fn);
      return true;
    };
}

bool AppOptions::parseArgs(int argc, char **argv)
{
  int i;

  for(i=1;i<argc;i++)
    if(argv[i][0]!=0)
      {
	if((strcasecmp(argv[i],"-v")==0)||(strcasecmp(argv[i],"-verbose")==0))
	  msglevel=2;
	else if((strcasecmp(argv[i],"-d")==0)||(strcasecmp(argv[i],"-debug")==0))
	  msglevel=3;
	else if((strcasecmp(argv[i],"-q")==0)||(strcasecmp(argv[i],"-quiet")==0))
	  {
	    msglevel=0;
	    errhelp=false;
	  }
	else if((strcasecmp(argv[i],"-n")==0)||(strcasecmp(argv[i],"-nohelp")==0))
	  errhelp=false;
	else if((strcasecmp(argv[i],"-h")==0)||(strcasecmp(argv[i],"-help")==0))
	  {
	    usage();
	    return false;
	  }
	else if(strncasecmp(argv[i],"-o=",3)==0)
	  {
	    if(!trySetOutName(&argv[i][3]))
	      return false;
	  }
	else if(strncasecmp(argv[i],"-outname=",9)==0)
	  {
	    if(!trySetOutName(&argv[i][9]))
	      return false;
	  }
	else if(argv[i][0]=='-')
	  {
	    printf("Unknown option '%s' ignored\n",argv[i]);
	  }
	else if(inputfname != NULL)
	  {
	    perror(0,ERROR_DUPLICATE_IFILE,"Multiple input files specified ('%s' and '%s')",inputfname,argv[i]);
	    return false;
	  }
	else
	  inputfname=argv[i];
      };
  return true;
}
