/*
 *  Part of rvglue source code,
 *  Copyright (C) 2000 Alexander Kroeller (alilein@gmx.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <stdio.h>
#include <stdlib.h>
#include "errdesc.h"
#include "appopts.h"

void describe_error(int errnum)
{
  if(appopt.errhelp)
    switch(errnum)
      {

	/********* ERRORS ********/

      case ERROR_ITOP_SYNTAX:
	printf("   (The input file contains something at top level that is neither a file name\n");
	printf("    nor one of the keywords like `global' or `create')\n");
	break;
      case ERROR_ITOP_FILETYPE:
	printf("   (You can insert .fin and .w files only. If you rename files, make sure that\n");
	printf("    their extensions are not changed.)\n");
	break;
      case ERROR_FILE_ROPEN:
	printf("   (A file could not be opened. Make sure that the file exists in the current\n");
	printf("    directory, is readable and has the correct name.)\n");
	break;
      case ERROR_EOF_IN_BLOCK:
	printf("   (Reached end of file while reading a block. This is likely due to a\n");
	printf("    forgotten closing bracket.)\n");
	break;
      case ERROR_EXPECTED_FNAME:
	printf("   (A filename was expected, but something else found.)\n");
	break;
      case ERROR_MAXFOPTS_EXCEEDED:
	printf("   (You exceeded an internal limit (MAX_FOPTS in this case). Increase that value\n");
	printf("    in app\\optkeep.h and recompile the program.)\n");
	break;
      case ERROR_DUPLICATE_IFILE:
	printf("   (You specified more than one file to be read. This may be because you forgot\n");
	printf("    the minus sign '-' of an command line option. Type 'rvglue -help' for a list\n");
	printf("    of supported options)\n");
	break;
      case ERROR_NO_IFILE:
	printf("   (You forgot to specify what command file you want me to process.\n");
	printf("    Type 'rvglue -help' for a list of supported options)\n");
	break;
      case ERROR_DUPLICATE_OFILE:
	printf("   (There are more than one '-o=...' and '-output=...' options.\n");
	printf("    Type 'rvglue -help' for a list of supported options)\n");
	break;
      case ERROR_EMPTY_OFILE:
	printf("   (There is a '-o=...' or '-output=...' option, but no filename is\n");
	printf("    following or there is a space character after the '=')\n");
	break;
      case ERROR_BEND_EXPECTED:
	printf("   (The block that is currently being read should be closed here, because\n");
	printf("    there is nothing that could stand here)\n");
	break;
      case ERROR_UNKNOWN_OPTION:
	printf("   (There is something in an option list that is none of the supported\n");
	printf("    options. Check for typos)\n");
	break;
      case ERROR_EXPECTED_KEY:
	printf("   (The program expected a legal value in this position, like a special\n");
	printf("    keyword or numerical value, but found something invalid.)\n");
	break;
      case ERROR_EXPECTED_TED_KEY:
	printf("   (The program expected a keyword for the 'tedelete' section, like\n");
	printf("    'pwall' or 'floor', but found something else. Check for typos.)\n");
	break;


	/************ WARNINGS **********/

      case ERROR_COMMENT_EOF:
	printf("   (Detected a comment that is never closed. Everything that follows the opening\n");
	printf("    bracket `[' is treated as comment.)\n");
	break;
      case ERROR_COMMENT_NOBEGIN:
	printf("   (Detected a `]' where no preceeding `[' was found. This is not fatal, but\n");
	printf("    may be a hint that you screwed up your comments in the input file and\n");
	printf("    get unexpected results or errors.)\n");
	break;
      case ERROR_EMPTY_FILEOPT:
	printf("   (An option set for a given instance name is missing.)\n");
	break;
      case ERROR_DUPLICATE_FILEOPT:
	printf("   (An instance name with options is defined multiple times. Every option set\n");
	printf("    except the first one is ignored.)\n");
	break;
      case ERROR_DUPLICATE_GLOBAL:
	printf("   (The input file contains more than one 'global' section. Only the first\n");
	printf("    is used, the other ones are discarded.)\n");
	break;
      case ERROR_USELESS_BLOCK:
	printf("   (There is a block indicated by brackets '(...)' where no one was expected\n");
	printf("    The block is thus useless and will be ignored)\n");
	break;

      default:
	printf("   (No help available)\n");
      };
}

void perror(int level, int errnum, const char *args, ...)
{
  va_list ap;

#ifdef _USE_VTCODES
  printf("\e[1m");
#endif
  if(errnum>0)
    {
      if(ERROR_IS_WARNING(errnum))
	printf("Warning: ");
      else
	printf("Error: ");
    }
  va_start(ap, args);  
  vprintf(args, ap);
  va_end(ap);
#ifdef _USE_VTCODES
  printf("\e[0m");
#endif
  printf("\n");

  if(errnum > 0)
    describe_error(errnum);
}

