/*
 *  Part of rvglue source code,
 *  Copyright (C) 2000 Alexander Kroeller (alilein@gmx.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include <rvtypes.h>
#include <print.h>

#include "finf.h"
#include "errdesc.h"
#include "readprm.h"



struct FIN_Data
{
  char fname[9];

  unsigned char rgb_model[3];
  unsigned char rgb_env[3];

  unsigned char _unknown[5];

  rvlong lodbias;

  RV_Vector position;
  rvfloat rot[3][3];
};




void modifyMesh(Mesh *mesh, FIN_Data *fdata)
{
  mesh->rotate(fdata->rot);
  mesh->translate(fdata->position.coord[0], fdata->position.coord[1],fdata->position.coord[2]);
}


int process_fin_file(const char *fname, OptionSetKeeper &keeper, World *world)
{
  FILE *f;
  rvlong count, i;
  FIN_Data fdata;
  Mesh *mesh;
  OptionSet *optset;
  char prmfname[20];

  f=fopen(fname,"rb");
  if(f==NULL)
    {
      perror(0, ERROR_FILE_ROPEN, "Cannot open `%s': %s",fname,strerror(errno));
      return 100;
    };

  lprintf(1,"Reading `%s'\n",fname);
  fread(&count,sizeof(rvlong),1,f);

  for(i=0;i<count;i++)
    {
      fread(&fdata, sizeof(FIN_Data),1,f);
      lprintf(2,"  Insert '%s.prm' at <%g,%g,%g>\n",fdata.fname,
	      fdata.position.coord[0],fdata.position.coord[1],fdata.position.coord[2]);
      strcpy(prmfname,fdata.fname);
      strcat(prmfname,".prm");
      mesh=create_prm_mesh(prmfname);
      if(mesh == NULL)
	{
	  fclose(f);
	  return 100;
	};

      modifyMesh(mesh,&fdata);
      optset=keeper.lookup(fdata.fname);
      optset->processMesh(mesh, world);
    };
  fclose(f);
  return 0;
}
