/*
 *  Part of rvglue source code,
 *  Copyright (C) 2000 Alexander Kroeller (alilein@gmx.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <stdio.h>
#include <stdlib.h>

#ifdef __SHOW_WIN_ERR
#include <windows.h>
#endif

#include <rvtypes.h>
#include <world.h>

#include "input.h"
#include "appopts.h"
#include "errdesc.h"

void check_env(void)
{
  if((sizeof(rvfloat)!=4) || (sizeof(rvshort)!=2) || (sizeof(rvlong)!=4))
    {
      printf("*** COMPILATION ERROR ***\n");
      printf("Recompile program for a 32-bit environment, or update\n");
      printf("the basic types in rvtypes.h to fit your environment!\n");
      exit(255);
    };
}

void welcome(void)
{
  lprintf(1,"  /-------------------------------------------\n");
  lprintf(1,"  | rvglue version %s, Copyright (C) 2000 Alexander Kroeller\n",APPVERSION);
  lprintf(1,"  | rvglue comes with ABSOLUTELY NO WARRANTY; for details see file 'GPL.txt'.\n");
  lprintf(1,"  | This is free software, and you are welcome to redistribute it\n");
  lprintf(1,"  | under certain conditions; see file 'GPL.txt' for details.\n");
  lprintf(1,"  \\-------------------------------------------\n");
}

#ifdef __SHOW_WIN_ERR
void noargs(void)
{
  MessageBox(NULL, "Items not added", "rvglue", MB_OK);
}
#endif

int main(int argc, char **argv)
{
  int res;
  World *world;

  check_env();

#ifdef __SHOW_WIN_ERR
  if(argc<2)
    noargs();
#endif


  if(!appopt.parseArgs(argc,argv))
    return 100;

  if(appopt.inputfname==NULL)
    {
      perror(0,ERROR_NO_IFILE,"No input file specified.");
      return 100;
    };

  welcome();

  world=new World;

  res=processInput(appopt.inputfname, world);
  if(res == 0)
    world->write(appopt.woutname,appopt.ncpoutname);

  delete world;

  return res;
}
