/*
 *  Part of rvglue source code,
 *  Copyright (C) 2000 Alexander Kroeller (alilein@gmx.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include <rvtypes.h>

#include "wf.h"
#include "errdesc.h"


int process_w_file(const char *fname, OptionSet &optset, World *world)
{
  FILE *f;
  rvlong count, i;
  RV_W_Mesh wmesh;
  Mesh *mesh;
  int lastp=-1;

  f=fopen(fname,"rb");
  if(f==NULL)
    {
      perror(0, ERROR_FILE_ROPEN, "Cannot open `%s': %s",fname,strerror(errno));
      return 100;
    };

  fread(&count,sizeof(rvlong),1,f);
  lprintf(1,"Reading `%s': %d meshes\n",fname,count);

  for(i=0;i<count;i++)
    {
      showPercent(2,i,count,lastp);

      fread(&wmesh, RV_W_MESH_SIZE,1,f);
      mesh=new Mesh(&wmesh,f);
      optset.processMesh(mesh, world);
    };
  doneShowPercent(2);
  fclose(f);
  return 0;
}
