/*
 *  Part of rvglue source code,
 *  Copyright (C) 2000 Alexander Kroeller (alilein@gmx.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef __MESH_H
#define __MESH_H

#include "rvtypes.h"
#include "polygon.h"
#include "vertex.h"
#include "surface.h"

class Mesh
{
public:
  Mesh(RV_PRM_Mesh*, FILE *);
  Mesh(RV_W_Mesh*, FILE *);
  Mesh();
  ~Mesh();

  void setTransparency(rvulong);

  void translate(rvfloat dx, rvfloat dy, rvfloat dz);
  void rotate(rvfloat[3][3]);

  void write_w(FILE *);

  void setTEColor(rvulong);

  void deleteTEParts(int);
  
  bool isDegenerated(void);

  void cleanupPolyList(void);

  int numPolys(void);

  void setSurfaceType(rvulong);
  rvulong surfaceType(void);

  Surface *createPolySurface(int pidx);

  /*  void add(Polygon);
  void add(PolyGroup &);

  void print(FILE *);
  void write(FILE *);
  void write_prm(FILE *);

  int size(void);

  //  void crstruct(RV_PolyGroup&, VertList &);

  Polygon &polygon(int); */

private:
  void load(FILE *);

  void calc_bounds(void);

  rvulong surfacetype;
  RV_W_Mesh data;
  Polygon **poly;
  Vertex **verts;
};



#endif
