/*
 *  Part of rvglue source code,
 *  Copyright (C) 2000 Alexander Kroeller (alilein@gmx.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <math.h>

#include "mesh.h"
#include "ncpwriter.h"
#include "lookup.h"


NCPFileWriter::NCPFileWriter(rvulong maxsz)
{
  cur_size=0;
  max_size=maxsz;
  slist=new (Surface*)[maxsz];
  grid=NULL;
}

NCPFileWriter::~NCPFileWriter()
{
  rvulong i;
  for(i=0;i<cur_size;i++)
    delete slist[i];
  delete slist;
  if(grid != NULL)
    delete grid;
}

bool NCPFileWriter::addSurface(Surface *s)
{
  if(cur_size < max_size)
    {
      slist[cur_size]=s;
      cur_size++;
      return true;
    }
  else
    {
      printf("AARRRRGGGGHHHH\n");
      return false;
    };
}

bool NCPFileWriter::addMesh(Mesh *m)
{
  int i;

  for(i=0;i<m->numPolys();i++)
    if(!addSurface(m->createPolySurface(i)))
      return false;
  return true;
}


int NCPFileWriter::write(const char *fname)
{
  FILE *f;
  rvulong i;
  rvushort sh_sz=cur_size;

  f=fopen(fname,"wb");
  if(f==NULL)
    return errno;

  fwrite(&sh_sz, sizeof(rvushort), 1, f);

  for(i=0;i<cur_size;i++)
    slist[i]->write(f);

  grid->write(f);

  fclose(f);
  return 0;
}


void NCPFileWriter::createGrid(void)
{
  rvulong i;

  grid=new LookupGrid(cur_size);
  for(i=0;i<cur_size;i++)
    slist[i]->expandGrid(grid);
  
  grid->doneSizing();

  for(i=0;i<cur_size;i++)
    slist[i]->enterIntoGrid(grid,i);
}
