/*
 *  Part of rvglue source code,
 *  Copyright (C) 2000 Alexander Kroeller (alilein@gmx.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef __RVTYPES_H
#define __RVTYPES_H

#include <math.h>

typedef float                rvfloat;   /* 32 bit */
typedef          short int   rvshort;   /* 16 bit */
typedef unsigned short int   rvushort;  /* 16 bit */
typedef                int   rvlong;    /* 32 bit */
typedef unsigned       int   rvulong;   /* 32 bit */



#define SQR(f) ((f)*(f))

#define EPSILON (0.0001)

#define FEQUAL(a,b)   ( fabs((a)-(b))<EPSILON )





struct RV_UV
{
  rvfloat     u; // Texture coords: 0<u,v<1
  rvfloat     v;
};
#define RV_UV_SIZE sizeof(RV_UV)





/*
 * RV_Vector
 *
 * simply three floats (x, y and z)
 *
 * class is 'Vector' (vector.h)
 */
struct RV_Vector
{
  rvfloat     coord[3];
};
#define RV_VECTOR_SIZE sizeof(RV_Vector)




struct RV_W_Mesh
{
  RV_Vector   ball_center;
  rvfloat     ball_radius;

  rvfloat     bbox_xlo, bbox_xhi;
  rvfloat     bbox_ylo, bbox_yhi;
  rvfloat     bbox_zlo, bbox_zhi;

  rvushort    num_polys;
  rvushort    num_vecs;
};
#define RV_W_MESH_SIZE sizeof(RV_W_Mesh)



struct RV_PRM_Mesh
{
  rvushort    num_polys;
  rvushort    num_vecs;
};
#define RV_PRM_MESH_SIZE sizeof(RV_PRM_Mesh)









/*
 * RV_Poly
 *
 * A three- or four-sided, flat polygon
 * with a texture.
 *
 * class is 'Polygon' (polygon.h)
 */
struct RV_Poly
{
  rvushort    type; // bitfield

  rvshort     texture; // 0=a.bmp, 1=b.bmp,..., -1=Solid colored

  rvushort    vertidx[4];
  rvulong     color[4];
  RV_UV       texcoord[4];
};
#define RV_POLY_SIZE sizeof(RV_Poly)






/*
 * RV_Vertex
 *
 * Two vectors: 
 *              'position' (x,y,z)
 *              'normal'   (u,v,w)
 *
 * class is 'Vertex' (vertex.h)
 */
struct RV_Vertex
{
  RV_Vector   pos;
  RV_Vector   normal;
};
#define RV_VERTEX_SIZE sizeof(RV_Vertex)




struct RV_FunnyBall
{
  RV_Vector   center;
  rvfloat     radius;
  
  rvulong     groupcnt;
};
#define RV_FUNNYBALL_SIZE sizeof(RV_FunnyBall)






struct RV_Plane
{
  RV_Vector  normal;
  rvfloat    distance;
};
#define RV_PLANE_SIZE sizeof(RV_Plane);




struct RV_Polyhedron
{
  rvlong   type;
  rvlong   surface;

  RV_Plane plane[5];

  rvfloat  bbox_xlo, bbox_xhi;
  rvfloat  bbox_ylo, bbox_yhi;
  rvfloat  bbox_zlo, bbox_zhi;
};
#define RV_POLYHEDRON_SIZE sizeof(RV_Polyhedron)




struct RV_LookupGrid
{
  rvfloat    x0;
  rvfloat    z0;

  rvfloat    x_size;
  rvfloat    z_size;

  rvfloat    raster_size;
};
#define RV_LOOKUPGRID_SIZE sizeof(RV_LookupGrid)


#endif
