/*
 *  Part of rvglue source code,
 *  Copyright (C) 2000 Alexander Kroeller (alilein@gmx.de)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef __VECTOR_H
#define __VECTOR_H

#include "rvtypes.h"

class Vector
{
public:
  Vector();
  Vector(RV_Vector*);    // copy from rv-mem-data
  Vector(const Vector &);      // copy from other vector

  Vector(Vector &, Vector &);  // be the normal vector to those
  Vector(rvfloat x, rvfloat y, rvfloat z); // spec. x,y,z

  Vector operator - (Vector &);
  Vector operator + (Vector &);

  Vector operator * (rvfloat scalefac); // scaling
  Vector operator * (Vector &);         // crossprod

  bool   operator == (Vector &);
  bool   operator != (Vector &);

  rvfloat length(void);
  bool  normalize(void);

  rvfloat scalarprod(Vector &);

  rvfloat &x(void); 
  rvfloat &y(void);
  rvfloat &z(void);
  rvfloat x(rvfloat);
  rvfloat y(rvfloat);
  rvfloat z(rvfloat);

  RV_Vector data;
};

#endif
